@@ -1,3 +1,4 @@
+from datetime import datetime
 from math import ceil
 from operator import attrgetter
 
@@ -13,14 +14,18 @@ from django.db.models import FileField, Value
 from django.db.models.functions import Lower, Now
 from django.test import (
     TestCase,
-    override_settings,
+    TransactionTestCase,
     skipIfDBFeature,
     skipUnlessDBFeature,
 )
+from django.test.utils import CaptureQueriesContext
+from django.utils import timezone
 
 from .models import (
     BigAutoFieldModel,
     Country,
+    DbDefaultModel,
+    DbDefaultPrimaryKey,
     FieldsWithDbColumns,
     NoFields,
     NullableFields,
@@ -212,12 +217,11 @@ class BulkCreateTests(TestCase):
 
     @skipUnlessDBFeature("has_bulk_insert")
     def test_large_batch_efficiency(self):
-        with override_settings(DEBUG=True):
-            connection.queries_log.clear()
+        with CaptureQueriesContext(connection) as ctx:
             TwoFields.objects.bulk_create(
                 [TwoFields(f1=i, f2=i + 1) for i in range(0, 1001)]
             )
-            self.assertLess(len(connection.queries), 10)
+            self.assertLess(len(ctx), 10)
 
     def test_large_batch_mixed(self):
         """
@@ -243,15 +247,14 @@ class BulkCreateTests(TestCase):
         Test inserting a large batch with objects having primary key set
         mixed together with objects without PK set.
         """
-        with override_settings(DEBUG=True):
-            connection.queries_log.clear()
+        with CaptureQueriesContext(connection) as ctx:
             TwoFields.objects.bulk_create(
                 [
                     TwoFields(id=i if i % 2 == 0 else None, f1=i, f2=i + 1)
                     for i in range(100000, 101000)
                 ]
             )
-            self.assertLess(len(connection.queries), 10)
+            self.assertLess(len(ctx), 10)
 
     def test_explicit_batch_size(self):
         objs = [TwoFields(f1=i, f2=i) for i in range(0, 4)]
@@ -289,6 +292,14 @@ class BulkCreateTests(TestCase):
         with self.assertNumQueries(ceil(len(objs) / max_batch_size)):
             Country.objects.bulk_create(objs, batch_size=max_batch_size + 1)
 
+    @skipUnlessDBFeature("has_bulk_insert")
+    def test_max_batch_size(self):
+        objs = [Country(name=f"Country {i}") for i in range(1000)]
+        fields = ["name", "iso_two_letter", "description"]
+        max_batch_size = connection.ops.bulk_batch_size(fields, objs)
+        with self.assertNumQueries(ceil(len(objs) / max_batch_size)):
+            Country.objects.bulk_create(objs)
+
     @skipUnlessDBFeature("has_bulk_insert")
     def test_bulk_insert_expressions(self):
         Restaurant.objects.bulk_create(
@@ -840,3 +851,67 @@ class BulkCreateTests(TestCase):
                 {"rank": 2, "name": "d"},
             ],
         )
+
+    @skipUnlessDBFeature("supports_expression_defaults")
+    def test_db_default_field_excluded(self):
+        # created_at is excluded when no db_default override is provided.
+        with self.assertNumQueries(1) as ctx:
+            DbDefaultModel.objects.bulk_create(
+                [DbDefaultModel(name="foo"), DbDefaultModel(name="bar")]
+            )
+        created_at_quoted_name = connection.ops.quote_name("created_at")
+        self.assertEqual(
+            ctx[0]["sql"].count(created_at_quoted_name),
+            1 if connection.features.can_return_rows_from_bulk_insert else 0,
+        )
+        # created_at is included when a db_default override is provided.
+        with self.assertNumQueries(1) as ctx:
+            DbDefaultModel.objects.bulk_create(
+                [
+                    DbDefaultModel(name="foo", created_at=timezone.now()),
+                    DbDefaultModel(name="bar"),
+                ]
+            )
+        self.assertEqual(
+            ctx[0]["sql"].count(created_at_quoted_name),
+            2 if connection.features.can_return_rows_from_bulk_insert else 1,
+        )
+
+    @skipUnlessDBFeature(
+        "can_return_rows_from_bulk_insert", "supports_expression_defaults"
+    )
+    def test_db_default_primary_key(self):
+        (obj,) = DbDefaultPrimaryKey.objects.bulk_create([DbDefaultPrimaryKey()])
+        self.assertIsInstance(obj.id, datetime)
+
+
+@skipUnlessDBFeature("supports_transactions", "has_bulk_insert")
+class BulkCreateTransactionTests(TransactionTestCase):
+    available_apps = ["bulk_create"]
+
+    def test_no_unnecessary_transaction(self):
+        with self.assertNumQueries(1):
+            Country.objects.bulk_create(
+                [Country(id=1, name="France", iso_two_letter="FR")]
+            )
+        with self.assertNumQueries(1):
+            Country.objects.bulk_create([Country(name="Canada", iso_two_letter="CA")])
+
+    def test_objs_with_and_without_pk(self):
+        with self.assertNumQueries(4):
+            Country.objects.bulk_create(
+                [
+                    Country(id=10, name="France", iso_two_letter="FR"),
+                    Country(name="Canada", iso_two_letter="CA"),
+                ]
+            )
+
+    def test_multiple_batches(self):
+        with self.assertNumQueries(4):
+            Country.objects.bulk_create(
+                [
+                    Country(name="France", iso_two_letter="FR"),
+                    Country(name="Canada", iso_two_letter="CA"),
+                ],
+                batch_size=1,
+            )
